@@ -1,24 +1,36 @@
 #ifndef KVAZAAR_H_
 #define KVAZAAR_H_
 /*****************************************************************************
-* This file is part of Kvazaar HEVC encoder.
-*
-* Copyright (C) 2013-2015 Tampere University of Technology and others (see
-* COPYING file).
-*
-* Kvazaar is free software: you can redistribute it and/or modify it under
-* the terms of the GNU Lesser General Public License as published by the
-* Free Software Foundation; either version 2.1 of the License, or (at your
-* option) any later version.
-*
-* Kvazaar is distributed in the hope that it will be useful, but WITHOUT ANY
-* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-* FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
-* more details.
-*
-* You should have received a copy of the GNU General Public License along
-* with Kvazaar.  If not, see <http://www.gnu.org/licenses/>.
-****************************************************************************/
+ * This file is part of Kvazaar HEVC encoder.
+ *
+ * Copyright (c) 2021, Tampere University, ITU/ISO/IEC, project contributors
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without modification,
+ * are permitted provided that the following conditions are met:
+ * 
+ * * Redistributions of source code must retain the above copyright notice, this
+ *   list of conditions and the following disclaimer.
+ * 
+ * * Redistributions in binary form must reproduce the above copyright notice, this
+ *   list of conditions and the following disclaimer in the documentation and/or
+ *   other materials provided with the distribution.
+ * 
+ * * Neither the name of the Tampere University or ITU/ISO/IEC nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION HOWEVER CAUSED AND ON
+ * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * INCLUDING NEGLIGENCE OR OTHERWISE ARISING IN ANY WAY OUT OF THE USE OF THIS
+ ****************************************************************************/
 
 /** 
  * \ingroup Control
@@ -27,6 +39,7 @@
  */
 
 #include <stdint.h>
+#include <stdio.h>
 
 
 #ifdef __cplusplus
@@ -74,13 +87,18 @@ extern "C" {
  */
 #define KVZ_DATA_CHUNK_SIZE 4096
 
+#ifndef KVZ_BIT_DEPTH
 #define KVZ_BIT_DEPTH 8
+#endif
+
 #if KVZ_BIT_DEPTH == 8
 typedef uint8_t kvz_pixel;
 #else
 typedef uint16_t kvz_pixel;
 #endif
 
+typedef int16_t kvz_pixel_im;  // For intermediate precision (interpolation/bipred).
+
 /**
  * \brief Opaque data structure representing one instance of the encoder.
  */
@@ -224,6 +242,20 @@ enum kvz_rc_algorithm
   KVZ_LAMBDA = 1,
   KVZ_OBA = 2,
 };
+
+enum kvz_file_format
+{
+  KVZ_FORMAT_AUTO = 0,
+  KVZ_FORMAT_Y4M = 1,
+  KVZ_FORMAT_YUV = 2
+};
+
+enum kvz_roi_format
+{
+  KVZ_ROI_TXT = 0,
+  KVZ_ROI_BIN = 1
+};
+
 // Map from input format to chroma format.
 #define KVZ_FORMAT2CSP(format) ((enum kvz_chroma_format)"\0\1\2\3"[format])
 
@@ -361,10 +393,9 @@ typedef struct kvz_config
   int32_t implicit_rdpcm; /*!< \brief Enable implicit residual DPCM. */
 
   struct {
-    int32_t width;
-    int32_t height;
-    int8_t *dqps;
-  } roi; /*!< \since 3.14.0 \brief Map of delta QPs for region of interest coding. */
+    char *file_path;
+    enum kvz_roi_format format;
+  } roi; /*!< \brief Specify delta QPs for region of interest coding. */
 
   unsigned slices; /*!< \since 3.15.0 \brief How to map slices to frame. */
 
@@ -434,6 +465,33 @@ typedef struct kvz_config
   uint8_t intra_bit_allocation;
 
   uint8_t clip_neighbour;
+
+  enum kvz_file_format file_format;
+
+  char *stats_file_prefix;
+  char *fast_coeff_table_fn;   /*!< \brief Pointer to fast coeff table filename */
+
+  /** \brief whether we're sampling TBs and their costs for fast cost
+   *         estimation training */
+  uint8_t rdo_cost_sampling_mode_on;
+
+  /** \brief whether we're running in normal mode, sampling TBs and their cost
+   *         for fast estimation training, or comparing estimator accuracy to
+   *         CABAC */
+  uint8_t fastrd_sampling_on;
+  uint8_t fastrd_accuracy_check_on;
+
+  char *fastrd_learning_outdir_fn;
+
+  /** \brief whether to try combining intra cus at the lower depth when search
+   *         is not performed at said depth*/
+  uint8_t combine_intra_cus;
+
+  uint8_t force_inter;
+
+  uint8_t intra_chroma_search;
+
+  uint8_t fast_bipred;
 } kvz_config;
 
 /**
@@ -465,6 +523,14 @@ typedef struct kvz_picture {
   enum kvz_chroma_format chroma_format;
 
   int32_t ref_pocs[16];
+
+  struct
+  {
+    int width;
+    int height;
+    int8_t *roi_array;
+  } roi;
+
 } kvz_picture;
 
 /**
@@ -709,6 +775,9 @@ typedef struct kvz_api {
    * the bitstream, length of the bitstream, the reconstructed frame, the
    * original frame and frame info in data_out, len_out, pic_out, src_out and
    * info_out, respectively. Otherwise, set the output parameters to NULL.
+   * 
+   * Region of interest (ROI) / delta QP map can be specified in the input
+   * picture's ROI field but only when a ROI file is not used.
    *
    * After passing all of the input frames, the caller should keep calling this
    * function with pic_in set to NULL, until no more data is returned in the
